# Pacakge 模型

Pacakge 模型部分介绍了 Serverless Package Model (SPM) 的建设模型，包括了：

- [模型的组成](#模型的组成)
- [模型的规范](#模型的规范)
    - [组件模型规范](#组件模型规范)
        - [组件模型元数据](#组件模型元数据)
            - [参数详解](#参数详解)
                - [Provider](#Provider)
                - [Category](#Category)
                - [Service](#Service)
                - [Properties](#Properties)
        - [组件模型代码规范](#组件模型代码规范)
    - [应用模型规范](#应用模型规范)
        - [应用模型元数据](#应用模型元数据)
            - [参数详解](#参数详解-1)
                - [Provider](#Provider-1)
                - [Category](#Category-1)
                - [Service](#Service-1)

## 模型的组成

Serverless Package Model 包括两个部分：

- Component Model：组件模型，即通过Serverless Devs，可以被应用所引用，并按照用户的输入，执行预定的功能。例如某个应用中引用了FC组件，那么此时，用户可以通过传入Deploy命令进行函数的部署，而这里的FC组件，则是需要建立在组件模型基础之上，即要符合组件的开发规范；
- Application Model：应用模型，即通过Serverless Devs，可以被初始化的应用案例，通常一个应用案例包括了一个yaml文件，在该文件中可以包括一个或多个组件来共同完成某个业务。这里所说的应用，就是需要建立在应用模型基础之上的，或者说是需要符合应用开发规范的；

## 模型的规范

### 组件模型规范

Component Model，即组件模型是需要通过指定的文件进行模型的规范和定义的。在这里，推荐的组件模型目录结构为：

```
|- src # 目录名字可以变更
|   └── 代码目录  
|- package.json: 需要定义好main   
|- publish.yaml: 项目的资源描述   
|- readme.md: 项目简介  
|- version.md: 版本更新内容
```

其中：

| 目录 | 必须 | 含义 |
| --- | --- | --- |
| src | 推荐存在 | 统一放置功能实现，当然也可以换成其他的名称，或者平铺到项目下，但是推荐通过src来做统一的存放 |
| package.json | 必须存在 |  Node.js的package.json，需要描述清楚组件的入口文件位置  |
| publish.yaml | 必须存在 | Serverless Devs Package的开发识别文档  |
| readme.md | 必须存在 | 对该组件的描述，或帮助文档信息  |
| version.md| 推荐存在 | 版本的描述，例如当前版本的更新内容等 |


#### 组件模型元数据

组件模型元数据将会在`publish.yaml`中进行描述，并在Serverless Registry和Serverless Devs开发者工具侧进行识别和引用。

`publish.yaml`文件的基本格式如下所示：

```yaml
Type: Component
Name: 名称
Provider:
  - 云厂商名称 
Version: 版本，例如0.0.1
Description: 简短的描述/介绍
HomePage: 项目首页地址
Tags: #标签详情
  - 部署函数
  - 部署组件
Category: 分类 # 基础云服务/Web框架/Web应用/人工智能/音视频处理/图文处理/监控告警/大数据/IoT/新手入门/其他
Service: # 使用的服务
  - Name: 服务名 # 函数计算/容器服务/镜像服务/消息队列/工作流/CDN/对象存储/表格存储/MNS/日志服务/API网关/数据库/解析服务/云应用/其他
    # Runtime: Python 3.6 如果服务是函数，还需要增加Runtime
    Authorities: #权限描述
      - 创建函数 # 所需要的权限
Commands: # 指令，格式为指令：指令描述，例如：
  deploy: 部署函数
  invoke: 调用函数
Properties:
  Region: # 参数
    Description: 参数描述
    Required: true # 参数必选，true/false
    Type: # 参数类型
      - String
```

##### 参数详解

| 目录 | 必须 | 结构 | 含义 |
| --- | --- | --- | --- |
| Type | 是 | String | 类型，包括Component和Application两个取值，此处取值Component |
| Name | 是 | String | 组件名称 |
| Provider | 是 | List<String> | 组件所支持的云厂商信息 |
| Version | 是 | String | 组件版本号，例如0.0.1 |
| Description | 是 | String | 组件描述（一句话的简短描述） |
| HomePage | 否 | String | 组件的主页，可以填写组件的仓库地址 |
| Tags | 否 | List<String> | 组件的标签 |
| Category | 是 | String | 组件的分类 |
| Service | 是 | Struct | 组件所需要的服务和相关的权限等描述，例如该组件需要函数计算，Serverless工作流等产品/服务作为支持 |
| Commands | 是 | Struct | 组件支持的命令 |
| Properties | 是 | Struct | 组件的参数描述，组件的属性定义 |

###### Provider

取值范围：`阿里云`, `百度智能云`, `华为云`, `腾讯云`, `AWS`, `Azure`, `Google Cloud`, `其它`

格式参考：
```yaml
Provider:
    - 阿里云
    - 百度智能云
```

###### Category

取值范围：`基础云服务`, `Web框架`, `全栈应用`, `人工智能`, `音视频处理`, `图文处理`, `监控告警`, `大数据`, `IoT`, `新手入门`, `其他`

格式参考：
```yaml
Category: 基础云服务
```

###### Service

取值范围：`函数计算`, `容器服务`, `镜像服务`, `消息队列`, `工作流`, `CDN`, `对象存储`, `表格存储`, `MNS`, `日志服务`, `API网关`, `数据库`, `解析服务`, `云应用`, `其他`

格式参考：
```yaml
Service: # 使用的服务
  - Name: 函数计算
    # Runtime: Python 3.6 如果服务是函数，还需要增加Runtime，取值包括：Node.JS, Python, PHP, Java, Go, 其它
    Authorities: #权限描述
      - 创建函数 # 所需要的权限
```

###### Properties

Properties是对组件的属性进行描述的，所以相对比较负责，主要包括：

| 目录 | 必须 | 结构 | 含义 |
| --- | --- | --- | --- |
| Description | 是 | String | 参数描述 |
| Required | 是 | String | 是否必须 |
| Type | 是 | Struct | 参数类型 |
| Example | 否 | String | 参数举例 |
| Default | 否 | String | 参数默认值 |

关于参数Type(类型)的额外定义：

基本参数类型：`String`, `Number`, `List`, `Enum`, `Struct`, `Boolean`, `Null`, `Any`    
复杂参数类型：`List<数据类型>`   

另外，当Type是List类型时，可以针对不同的元素做别名：`数据类型[别名]`

格式参考：
- 简单格式：
    ```yaml
    Properties:
      Region: # 参数
        Description: 参数描述
        Required: true # 参数必选，true/false
        Type: # 参数类型
          - String
    ```
  用户侧表现是：
   ```yaml
   Region: cn-hangzhou
   ```
- 复杂格式：
    ```yaml
    Properties:
      Region: # 参数
        Description: 地区
        Required: true # 参数必选，true/false
        Type: # 参数类型
          - String[简单配置]
          - List<String>[多地域配置]
          - Struct[分别配置]: 
              APIGW:
                Description: API网关部署的地区
                Required: true 
                Type: 
                  - String
              Function:
                Description: 函数部署的地区
                Required: true 
                Type: 
                  - String
    ```
  用户侧表现是：
  - 当Type为String[简单配置]时：
    ```yaml
    Region: cn-hangzhou
    ```
  - 当Type为List<String>[多地域配置]时：
    ```yaml
    Region: 
      - cn-hangzhou
      - cn-beijing
    ```
  - 当Type为Struct[分别配置]时：
    ```yaml
    Region: 
      APIGW: cn-beijing
      Function: cn-shanghai
    ```    
    
    

#### 组件模型代码规范

在组件模型中，代码组成规范有两个部分：
- `package.json`中需要描述清楚入口文件所在地址；例如`{"main": "./dist/index.js"}`；
- 在代码中实现对应的用户方法。例如Package开发者希望用户可以通过deploy命令，进行项目的部署，那么就可以实现一个deploy的方法，并在方法内实现对应的部署能力；

关于代码规范部分，可以参考如下案例：

```typescript
import logger from './common/logger';
import { InputProps } from './common/entity';

export default class ComponentDemo {
  /**
   * demo 实例
   * @param inputs
   * @returns
   */
  public async test(inputs: InputProps) {
    logger.debug(`input: ${JSON.stringify(inputs.props)}`);
    logger.info('command test');
    return { hello: 'world' };
  }
}
```

其中入参`inputs`的结构为：

```json
{
    "command": "", 
    "project": {
        "projectName": "", 
        "component": "",
        "provider": "",
        "access": ""
    },
    "credentials": {},
    "prop": {},
    "args": "",
    "argsObj": []
}
```

| 目录 | 含义 |
| --- | --- | 
| command | 用户所执行的命令 | 
| project | 用户的项目基本信息 | 
| credentials | 用户的密钥信息 | 
| prop | 用户配置的属性/参数 |
| args| 用户传递的参数（字符串形式） | 
| argsObj| 用户传递的参数（解析后的，以数组形式传递） |


在上面的案例代码中，可以看到有一个test方法，该方法就是功能实现的方法。此时当用户使用test命令时，系统就会携带参数调用该方法。以一个真实案例作为举例说明：

该组件名为`hexo`，组件核心代码如上所示，具备一个test方法，此时用户侧的Yaml为：

```yaml
edition: 1.0.0        #  命令行YAML规范版本，遵循语义化版本（Semantic Versioning）规范
name: FullStack       #  项目名称
access: xxx-account1  #  秘钥别名

services:
  HexoComponent:
    component: hexo
    props:
      region: 'cn-hangzhou'
      codeUri: './src'
```

当用户执行`s test mytest -a -b abc`，此时，组件代码中的`test`方法，收到的`inputs`参数实际上是：

```json
{
    "command": "test", 
    "project": {
        "projectName": "HexoComponent", 
        "component": "hexo",
        "provider": "alibaba",
        "access": "release"
    },
    "credentials": {
        "AccountID": "********",
        "AccessKeyID": "********",
        "AccessKeySecret": "********"
    },
    "prop": {
        "Region": "cn-hangzhou",
        "CodeUri": "./src"
    },
    "args": "mytest -a -b abc",
    "argsObj": [
      "mytest", "-a", "-b", "abc"
    ]
}
```

此时test方法会打印日志信息等，并返回最终的结果给命令行工具：`{ "hello": "world" }`

### 应用模型规范

Application Model，即组件模型是需要通过指定的文件进行模型的规范和定义的。在这里，推荐的组件模型目录结构为：

```
|- src # 目录名字不可以变更
|   └── 应用目录
|   └── s.yml: 应用描述文件
|- publish.yaml: 项目的资源描述   
|- readme.md: 项目简介  
|- version.md: 版本更新内容
```

其中：

| 目录 | 必须 | 含义 |
| --- | --- | --- |
| src | 必须存在 | 应用所在目录 |
| s.yml | 必须存在 | 应用的资源描述Yaml，需要符合该应用对应的publish，yaml规范 |
| publish.yaml | 必须存在 | Serverless Devs Package的开发识别文档  |
| readme.md | 必须存在 | 对该组件的描述，或帮助文档信息  |
| version.md| 推荐存在 | 版本的描述，例如当前版本的更新内容等 |

#### 应用模型元数据

应用模型元数据将会在`publish.yaml`中进行描述，并在Serverless Registry和Serverless Devs开发者工具侧进行识别和初始化。

`publish.yaml`文件的基本格式如下所示：

```yaml
Type: Application
Name: 名称
Provider:
  - 云厂商名称 
Version: 版本，例如0.0.1
Description: 简短的描述/介绍
HomePage: 项目首页地址
Tags: #标签详情
  - 部署函数
  - 部署组件
Category: 分类 # 基础云服务/Web框架/Web应用/人工智能/音视频处理/图文处理/监控告警/大数据/IoT/新手入门/其他
Service: # 使用的服务
  - Name: 服务名 # 函数计算/容器服务/镜像服务/消息队列/工作流/CDN/对象存储/表格存储/MNS/日志服务/API网关/数据库/解析服务/云应用/其他
    # Runtime: Python 3.6 如果服务是函数，还需要增加Runtime
    Authorities: #权限描述
      - 创建函数 # 所需要的权限
```

##### 参数详解

| 目录 | 必须 | 结构 | 含义 |
| --- | --- | --- | --- |
| Type | 是 | String | 类型，包括Component和Application两个取值，此处取值Application |
| Name | 是 | String | 组件名称 |
| Provider | 是 | List<String> | 组件所支持的云厂商信息 |
| Version | 是 | String | 组件版本号，例如0.0.1 |
| Description | 是 | String | 组件描述（一句话的简短描述） |
| HomePage | 否 | String | 组件的主页，可以填写组件的仓库地址 |
| Tags | 否 | List<String> | 组件的标签 |
| Category | 是 | String | 组件的分类 |
| Service | 是 | Struct | 组件所需要的服务和相关的权限等描述，例如该组件需要函数计算，Serverless工作流等产品/服务作为支持 |


###### Provider

取值范围：`阿里云`, `百度智能云`, `华为云`, `腾讯云`, `AWS`, `Azure`, `Google Cloud`, `其它`

格式参考：
```yaml
Provider:
    - 阿里云
    - 百度智能云
```

###### Category

取值范围：`基础云服务`, `Web框架`, `全栈应用`, `人工智能`, `音视频处理`, `图文处理`, `监控告警`, `大数据`, `IoT`, `新手入门`, `其他`

格式参考：
```yaml
Category: 基础云服务
```

###### Service

取值范围：`函数计算`, `容器服务`, `镜像服务`, `消息队列`, `工作流`, `CDN`, `对象存储`, `表格存储`, `MNS`, `日志服务`, `API网关`, `数据库`, `解析服务`, `云应用`, `其他`

格式参考：
```yaml
Service: # 使用的服务
  - Name: 函数计算
    # Runtime: Python 3.6 如果服务是函数，还需要增加Runtime，取值包括：Node.JS, Python, PHP, Java, Go, 其它
    Authorities: #权限描述
      - 创建函数 # 所需要的权限
```

> 特殊格式：在应用模型中，需要存在`src/s.yaml`文件，作为Serverless Devs识别和使用的资源、行为描述文件，在该文件中，可能涉及到部分内容是需要用户进行填写的，例如用户的密钥名字，用户部署业务的地域等。此时可以参考：
> - `"{{ access }}"` ：直接提醒用户需要输入access这样的一个参数，作为Yaml中所必须的参数；
> - `'{{ bucket | alibaba oss bucket }}'` ： ：直接提醒用户需要输入bucket这样的一个参数，作为Yaml中所必须的参数，并以`|`之后的内容"alibaba oss bucket"作为解释这个参数的含义；
> 例如，在某应用的`s.yaml`中表现为：
> ```yaml
> edition: 1.0.0
> access: "{{ access }}"
> 
> services:
>   website-starter:
>    component: devsapp/website
>     actions:
>       pre-deploy:
>         - run: npm install
>           path: ./
>         - run: npm run build
>           path: ./
>     props:
>       bucket: '{{ bucket | alibaba oss bucket }}'
>       src:
>         codeUri: ./
>         publishDir: ./build
>         index: index.html
>       region: cn-hangzhou
>       hosts:
>         - host: auto
> ```
